@@ -2214,3 +2214,12 @@ void gegl_node_progress (GeglNode *node,
     g_idle_add (delayed_emission, closure);
   }
 }
+
+const char *gegl_operation_get_op_version (const char *op_name)
+{
+  const gchar *ret = gegl_operation_get_key (op_name, "op-version");
+  if (!ret)
+    ret = "0:0";
+  return ret;
+}
+
